<?php
/**
 * Novalnet Callback Script for Ubercart
 *
 * NOTICE 
 *
 * This script is used for real time capturing of parameters passed 
 * from Novalnet AG after Payment processing of customers.
 *
 * This script is only free to the use for Merchants of Novalnet AG
 *
 * If you have found this script useful a small recommendation as well
 * as a comment on merchant form would be greatly appreciated.
 *
 * Please contact sales@novalnet.de for enquiry or info
 *
 * ABSTRACT: This script is called from Novalnet, as soon as a payment 
 * done for payment methods, e.g. Prepayment, Invoice.
 * An email will be sent if an error occurs
 *
 *
 * @category   Novalnet
 * @package    Novalnet
 * @version    1.0
 * @copyright  Copyright (c) 2012 Novalnet AG. (http://www.novalnet.de)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @notice     1. This script must be placed in Ubercart root folder
 *                to avoid rewrite rules (mod_rewrite)
 *             2. You have to adapt the value of all the variables
 *                commented with 'adapt ...'
 *             3. Set $test/$debug to false for live system
*/
define('DRUPAL_ROOT', getcwd());
 require_once 'includes/bootstrap.inc';
 drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
//(loads everything, but doesn't render anything)
require_once('includes/module.inc');

//Table Names
define('ORDER_TABLE', 'commerce_order');
define('MSG_TABLE', 'commerce_payment_transaction');

//Variable Settings
$debug        		  = true; //false|true; adapt: set to false for go-live
$test           	  = true; //false|true; adapt: set to false for go-live
$lineBreak     		  = empty($_SERVER['HTTP_HOST'])? PHP_EOL: '<br />';
$addSubsequentTidToDb = true;//whether to add the new tid to db; adapt if necessary

// Order State/Status Settings
          /*   Standard Types of Status:
				~~~~~~~~~~~~~~~~~~~~~~~
				processing
				completed
				canceled	

          */
$orderState  = "completed"; //Note: Indicates Payment accepted.

//Security Setting; only this IP is allowed for call back script
$ipAllowed = '195.143.189.210'; //Novalnet IP, is a fixed value, DO NOT CHANGE!!!!!

//Reporting Email Addresses Settings
$shopInfo      = 'Drupal Commerce'.$lineBreak; //manditory;adapt for your need
$mailHost      = 'mail.novalnet.de';//adapt
$mailPort      = 25;//adapt
$emailFromAddr = '';//sender email addr., manditory, adapt it
$emailToAddr   = '';//recipient email addr., manditory, adapt it
$emailSubject  = 'Novalnet Callback Script Access Report'; //adapt if necessary; 
$emailBody     = '';//Email text, adapt
$emailFromName = ""; // Sender name, adapt
$emailToName   = ""; // Recipient name, adapt

//Parameters Settings
$hParamsRequired = array(
  'vendor_id'    => '',
  'tid'          => '',
  'payment_type' => '',
  'status'       => '',
  'amount'       => '',
  'tid_payment'  => '');

$hParamsTest = array(
  'vendor_id'    => '4',
  'status'       => '100',
  'amount'       => '1000',//must be avail. in shop database; 850 = 8.50
  'payment_type' => 'INVOICE_CREDIT',
  'tid_payment'  => '12547400001816383',//orig. tid; must be avail. in shop database
  'tid'          => '12535700003115268',//subsequent tid, from Novalnet backend; can be a fake for test
  );

//Test Data Settings
if ($test){
  $_REQUEST      = $hParamsTest;
  $emailFromName = "Novalnet"; // Sender name, adapt
  $emailToName   = "Novalnet"; // Recipient name, adapt
  $emailFromAddr = 'test@novalnet.de';//manditory for test; adapt
  $emailToAddr   = 'test@novalnet.de';//manditory for test; adapt
  $emailSubject  = $emailSubject.' - TEST';//adapt
}

// ################### Main Prog. ##########################
try {
  //Check Params
  if (checkIP($_REQUEST)){
    if (checkParams($_REQUEST)){
	  //Get Order ID and Set New Order Status
	  getIncrementId($_REQUEST);
    }
  }
	
  if (!$emailBody){
    $emailBody .= 'Novalnet Callback Script called for StoreId Parameters: '.print_r($_POST, true).$lineBreak;
    $emailBody .= 'Novalnet callback succ. '.$lineBreak;
    $emailBody .= 'Params: '.print_r($_REQUEST, true).$lineBreak;
  }
 }catch(Exception $e){
   $emailBody .= "Exception catched: $lineBreak\$e:".$e->getMessage().$lineBreak;
 }

if ($emailBody){
  if (!sendEmail($emailBody)){
    if ($debug){
      echo "Mailing failed!".$lineBreak;
      echo "This mail text should be sent: ".$lineBreak;
      echo $emailBody;
    }
  }
}

// ############## Sub Routines #####################
function sendEmail($emailBody){
  global $lineBreak, $debug, $test, $emailFromAddr, $emailToAddr, $emailFromName, $emailToName, $emailSubject, $shopInfo, $mailHost, $mailPort;
  //Send Email
   $emailBodyT = str_replace('<br />', PHP_EOL, $emailBody);

 ini_set('SMTP', $mailHost);
  ini_set('smtp_port', $mailPort);

	header('Content-Type: text/html; charset=iso-8859-1');
	$headers = 'From: ' . $emailFromAddr . "\r\n";
	
  try {
    if ($debug){
      echo __FUNCTION__.': Sending Email suceeded!'.$lineBreak;
    }
	$sendmail = mail($emailToAddr, $emailSubject, $emailBodyT, $headers);
  }
  catch(Exception $e) {
    if ($debug) {echo 'Email sending failed: '.$e->getMessage();}
    return false;
  }
  if ($debug){
    echo 'This text has been sent:'.$lineBreak.$emailBody;
  }
  return true;
}
function checkParams($_request){
  global $lineBreak, $hParamsRequired, $emailBody;
  $error = false;
  $emailBody = '';
  if(!$_request){
    $emailBody .= 'No params passed over!'.$lineBreak;
    return false;
  }elseif($hParamsRequired){
    foreach ($hParamsRequired as $k=>$v){
      if (empty($_request[$k])){
        $error = true;
        $emailBody .= 'Required param ('.$k.') missing!'.$lineBreak;
      }
    }
    if ($error){
      return false;
    }
  }

  //Only Payment Type 'INVOICE_CREDIT' allowed; Otherwise you have to adapt the logic
  if (!empty($_request['payment_type']) and 'INVOICE_CREDIT' != strtoupper($_request['payment_type'])) {
		// Nothing to do
    $emailBody .= "Novalnet callback received. But payment_type != INVOICE_CREDIT (".$_request['payment_type'].")$lineBreak";
		return false;
	}

	if(!empty($_request['status']) and 100 != $_request['status']) {
		$emailBody .= 'The status codes [' . $_request['status'] . '] is not valid: Only 100 is allowed.' . "$lineBreak$lineBreak".$lineBreak;
    return false;
	}
  return true;
}
function getIncrementId($_request){
	global $lineBreak, $tableOrderPayment, $tableOrder, $emailBody, $debug, $oDB, $orderDetails;
	$orderDetails = array();

  if (!empty($_request['order_no'])){
    return $_request['order_no'];
  }elseif(!empty($_request['order_id'])){
    return $_request['order_id'];
  }
	$iOrderId  = 0;
	$mysql = db_query("SELECT order_id from ".MSG_TABLE." WHERE message LIKE '%".$_request['tid_payment']."%'")->fetchAll();
    $commerce_order = array_pop($mysql);
   if(!$commerce_order) {
		$emailBody .= 'The original order not found in the shop database table (`'.MSG_TABLE.'`);';
		//$emailBody .= 'Reason: '.$e->getMessage().$lineBreak.$lineBreak;
		//$emailBody .= 'Query : '.$qry.$lineBreak.$lineBreak;
		throw new Exception('0950101'); // Error in executing the query.
		return false;		
	} else {
		$commerce_orderId  = $commerce_order->order_id;
	}

	$orderDetails = commerce_order_load($commerce_orderId);	
	$wrapper = entity_metadata_wrapper('commerce_order', $orderDetails);
	
 if ($debug) {echo'Order Details:<pre>'; print_r($orderDetails);echo'</pre>';}

  if(empty($commerce_orderId) or !$orderDetails){
    $emailBody .= 'increment_id n/a'.$lineBreak;
    return false;
  }
 
  $order_total = $wrapper->commerce_order_total->amount->value();
  $amount  = $_request['amount'];
  $_amount = $order_total;
  //echo $_amount;exit;
	
  if(!$_amount || (intval("$_amount") != intval("$amount"))) {
    $emailBody .= "The order amount ($_amount) does not match with the request amount ($amount)$lineBreak$lineBreak";
    return false;
  }

  $paymentType = $orderDetails->data['payment_method'];
  $arr = explode("|", $paymentType);
  $paymentType = $arr[0];

   if(!in_array($paymentType, array('commerce_novalnet_invoice', 'commerce_novalnet_prepayment'))) {
   $emailBody .= "The order payment type ($paymentType) is not Prepayment!$lineBreak$lineBreak";

   return false;
  }

  setOrderStatus($commerce_orderId, $orderDetails);
}

function setOrderStatus ($incrementId, $orderDetails) {
  global $lineBreak, $createInvoice, $emailBody, $orderStatus, $orderState, $tableOrderPayment, $addSubsequentTidToDb, $oDB;
 
  if ($incrementId) {
	
      if ($addSubsequentTidToDb){
		$updatecomments = db_query("UPDATE {".MSG_TABLE."} set message = CONCAT(message,' Novalnet Callback Script executed successfully. The subsequent TID: (".$_REQUEST["tid"].") on ".date("Y-m-d H:i:s")."') , status = '".$orderState."' WHERE order_id=".$incrementId);
		$updateOrderStatus = db_query("UPDATE {".ORDER_TABLE."} set status = '".$orderState."' WHERE order_id=".$incrementId);	
      }
  } else {
    $emailBody .= "Novalnet Callback: No order for Increment-ID $incrementId found.";
    return false;
  }
  $emailBody .= "succeeded.";
  return true;
}
	function getRealIpAddr() {
	if(isPublicIP($_SERVER['HTTP_X_FORWARDED_FOR'])) return $_SERVER['HTTP_X_FORWARDED_FOR'];
	if($iplist=explode(',', $_SERVER['HTTP_X_FORWARDED_FOR'])) {
		if(isPublicIP($iplist[0])) return $iplist[0];
	}
	if (isPublicIP($_SERVER['HTTP_CLIENT_IP'])) return $_SERVER['HTTP_CLIENT_IP'];
	if (isPublicIP($_SERVER['HTTP_X_CLUSTER_CLIENT_IP'])) return $_SERVER['HTTP_X_CLUSTER_CLIENT_IP'];
	if (isPublicIP($_SERVER['HTTP_FORWARDED_FOR']) ) return $_SERVER['HTTP_FORWARDED_FOR'];
	return $_SERVER['REMOTE_ADDR'];
	}
	
	function isPublicIP($value) {
		if(!$value || count(explode('.',$value))!=4) return false;
		return !preg_match('~^((0|10|172\.16|192\.168|169\.254|255|127\.0)\.)~', $value);
	}
	
function checkIP($_REQUEST){
  global $lineBreak, $ipAllowed, $test, $emailBody;
  if ($test){ $ipAllowed = getRealIpAddr(); }
  $callerIp  = $_SERVER['REMOTE_ADDR'];

  if($ipAllowed != $callerIp) {
    $emailBody .= 'Unauthorised access from the IP [' . $callerIp . ']' .$lineBreak.$lineBreak;
    $emailBody .= 'Request Params: ' . print_r($_REQUEST, true);
    return false;
  }
  return true;
}

?>